// Copyright (c) Microsoft. All rights reserved.
// Licensed under the MIT license. See LICENSE file in the project root for full license information.
'use strict';
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var azure_iot_http_base_1 = require("azure-iot-http-base");
var azure_iot_provisioning_device_1 = require("azure-iot-provisioning-device");
var azure_iot_provisioning_device_2 = require("azure-iot-provisioning-device");
var dbg = require("debug");
var debug = dbg('azure-iot-provisioning-device-http:Http');
var _defaultHeaders = {
    'Accept': 'application/json',
    'Content-Type': 'application/json; charset=utf-8'
};
/**
 * Transport used to provision a device over HTTP.
 */
var Http = (function (_super) {
    __extends(Http, _super);
    /**
     * @private
     */
    /* Codes_SRS_NODE_PROVISIONING_HTTP_18_001: [ The `Http` constructor shall accept the following properties:
    - `idScope` - the ID Scope value for the provisioning service
    - `httpBase` - an optional test implementation of azure-iot-http-base ] */
    function Http(httpBase) {
        var _this = _super.call(this) || this;
        _this._config = {};
        _this._httpBase = httpBase || new azure_iot_http_base_1.Http();
        _this._config.pollingInterval = azure_iot_provisioning_device_1.ProvisioningDeviceConstants.defaultPollingInterval;
        _this._config.timeoutInterval = azure_iot_provisioning_device_1.ProvisioningDeviceConstants.defaultTimeoutInterval;
        return _this;
    }
    Http.prototype.setAuthentication = function (auth) {
        this._auth = auth;
    };
    /**
     * @private
     *
     */
    Http.prototype.setTransportOptions = function (options) {
        var _this = this;
        [
            'pollingInterval',
            'timeoutInterval'
        ].forEach(function (optionName) {
            if (options.hasOwnProperty(optionName)) {
                _this._config[optionName] = options[optionName];
            }
        });
    };
    /**
     * private
     */
    Http.prototype.cancel = function (callback) {
        // Nothing to do.
        callback();
    };
    /**
     * private
     */
    Http.prototype.registrationRequest = function (request, callback) {
        var _this = this;
        this._restApiClient = new azure_iot_http_base_1.RestApiClient({ 'host': request.provisioningHost, 'x509': this._auth }, azure_iot_provisioning_device_1.ProvisioningDeviceConstants.userAgent, this._httpBase);
        var requestBody = { registrationId: request.registrationId };
        /* update Codes_SRS_NODE_PROVISIONING_HTTP_18_009: [ `register` shall PUT the registration request to 'https://global.azure-devices-provisioning.net/{idScope}/registrations/{registrationId}/register' ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_005: [ The registration request shall include the current `api-version` as a URL query string value named 'api-version'. ] */
        var path = '/' + request.idScope + '/registrations/' + request.registrationId + '/register?api-version=' + azure_iot_provisioning_device_1.ProvisioningDeviceConstants.apiVersion;
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_008: [ If `forceRegistration` is specified, the registration request shall include this as a query string value named 'forceRegistration' ] */
        if (request.forceRegistration) {
            path += '&forceRegistration=true';
        }
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_006: [ The registration request shall specify the following in the Http header:
          Accept: application/json
          Content-Type: application/json; charset=utf-8 ] */
        var httpHeaders = JSON.parse(JSON.stringify(_defaultHeaders));
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_007: [ If an `auth` string is specifed, it shall be URL encoded and included in the Http Authorization header. ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_011: [ If the registration request times out, `register` shall call the `callback` with the lower level error] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_012: [ If the registration response contains a body, `register` shall deserialize this into an object. ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_013: [ If registration response body fails to deserialize, `register` will throw an `SyntaxError` error. ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_014: [ If the registration response has a failed status code, `register` shall use `translateError` to translate this to a common error object and pass this into the `callback` function along with the deserialized body of the response. ] */
        debug('submitting PUT for ' + request.registrationId + ' to ' + path);
        debug(JSON.stringify(requestBody));
        this._restApiClient.executeApiCall('PUT', path, httpHeaders, requestBody, this._config.timeoutInterval, function (err, result, response) {
            if (err) {
                debug('error executing PUT: ' + err.toString());
                callback(err, result, response);
            }
            else {
                debug('PUT response received:');
                debug(JSON.stringify(result));
                if (response.statusCode < 300) {
                    callback(null, result, response, _this._config.pollingInterval);
                }
                else {
                    callback(azure_iot_provisioning_device_2.translateError('PUT operation returned failure', response.statusCode, result, response));
                }
            }
        });
    };
    /**
     * private
     */
    Http.prototype.queryOperationStatus = function (request, operationId, callback) {
        var _this = this;
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_022: [ operation status request polling shall be a GET operation sent to 'https://global.azure-devices-provisioning.net/{idScope}/registrations/{registrationId}/operations/{operationId}' ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_037: [ The operation status request shall include the current `api-version` as a URL query string value named 'api-version'. ] */
        var path = '/' + request.idScope + '/registrations/' + request.registrationId + '/operations/' + operationId + '?api-version=' + azure_iot_provisioning_device_1.ProvisioningDeviceConstants.apiVersion;
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_020: [ The operation status request shall have the following in the Http header:
          Accept: application/json
          Content-Type: application/json; charset=utf-8 ] */
        var httpHeaders = JSON.parse(JSON.stringify(_defaultHeaders));
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_021: [ If an `auth` string is specifed, it shall be URL encoded and included in the Http Authorization header of the operation status request. ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_023: [ If the operation status request times out, `register` shall stop polling and call the `callback` with with the lower level error ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_024: [ `register` shall deserialize the body of the operation status response into an object. ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_025: [ If the body of the operation status response fails to deserialize, `register` will throw a `SyntaxError` error. ] */
        /* Codes_SRS_NODE_PROVISIONING_HTTP_18_026: [ If the operation status response contains a failure status code, `register` shall stop polling and call the `callback` with an error created using `translateError`. ] */
        debug('submitting status GET for ' + request.registrationId + ' to ' + path);
        this._restApiClient.executeApiCall('GET', path, httpHeaders, {}, this._config.timeoutInterval, function (err, result, response) {
            if (err) {
                debug('error executing GET: ' + err.toString());
                callback(err, result, response);
            }
            else {
                debug('GET response received:');
                debug(JSON.stringify(result));
                if (response.statusCode < 300) {
                    callback(null, result, response, _this._config.pollingInterval);
                }
                else {
                    callback(azure_iot_provisioning_device_2.translateError('GET operation returned failure', response.statusCode, result, response));
                }
            }
        });
    };
    return Http;
}(events_1.EventEmitter));
exports.Http = Http;
// The following are legacy requirements.  The used to be handled by http.ts, but they've been moved into transport_state_machine.ts.
// We're keeping these here mostly for the Http tests, which now overlap with the TransportStateMachine tests, but we're keeping
// the tests here for the extra test coverage.
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_016: [ If the registration response has a success code with a 'status' of 'Assigning', `register` shall fire an `operationStatus` event with the deserialized body ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_017: [ If the registration response has a success code with a 'status' of 'Assigning', `register` shall start polling for operation updates ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_015: [ If the registration response has a success code with a 'status' of 'Assigned', `register` call the `callback` with `err` == `null` and result `containing` the deserialized body ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_029: [ If the registration response has a success code with a 'status' that is any other value', `register` shall call the callback with a `SyntaxError` error. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_036: [ `register` shall call the `callback` with an `InvalidOperationError` if it is called while a previous registration is in progress. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_018: [ `register` shall poll for operation status every `operationStatusPollingInterval` milliseconds ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_036: [ `register` shall call the `callback` with an `InvalidOperationError` if it is called while a previous registration is in progress. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_028: [ If the operation status response contains a success status code with a 'status' that is 'Assigning', `register` shall fire an `operationStatus` event with the deserialized body and continue polling. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_027: [ If the operation status response contains a success status code with a 'status' of 'Assigned', `register` shall stop polling and call the `callback` with `err` == null and the body containing the deserialized body. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_030: [ If the operation status response has a success code with a 'status' that is any other value, `register` shall call the callback with a `SyntaxError` error and stop polling. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_036: [ `register` shall call the `callback` with an `InvalidOperationError` if it is called while a previous registration is in progress. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_035: [ disconnect will cause polling to cease ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_031: [ If `disconnect` is called while the registration request is in progress, `register` shall call the `callback` with an `OperationCancelledError` error. ] */
/* Codes_SRS_NODE_PROVISIONING_HTTP_18_033: [ If `disconnect` is called while the register is waiting between polls, `register` shall call the `callback` with an `OperationCancelledError` error. ] */
//# sourceMappingURL=http.js.map